{ ----------------------------------------------------------------------

  Copyright (c) 1997 by Martin Zahl
  eMail: Martin.Zahl@Informatik.uni-oldenburg.de

  The Idea of this component was to be able to save the configuration in the
  registry OR!! an inifile. So that you can select the destination without
  any changes in the source.
  I'm using the methods from TINIFile, because they have the same
  parameters as TRegistry "+ the Section-Field".
  So you can use the same method for both: TINIFile and TRegistry

  Version 1.01
  - ## Changes from Zdenek Petruj ##
  - He implements a greater exception-handling !!
  - Thanks to him !
  - new: function ValueExists(Section:string;Ident:string): Boolean;
  - new event: TMyRegistryError
  Version 1.10
  - new: property SaveNewKey : boolean;
    writes a key if he does not exist on reading
  - updated methods: open, close
    <>Nil checking implemented
  ---------------------------------------------------------------------- }

unit MyRegistry;

interface

uses
  Windows, Messages, SysUtils, Classes, Graphics, Controls, Forms, Dialogs,
  Registry, INIFiles, DsgnIntf;

const
  // Now you can download the newest version only by pressing the button
  // at "Homepage" !!
  MyHomepage = 'http://www.Informatik.Uni-Oldenburg.de/~mzahl';
type
  TMyRegistryError = procedure(Sender: TObject;IsBox:Boolean; EMessage : String) of object;
  HKeys = (kHKEY_CLASSES_ROOT, kHKEY_CURRENT_USER, kHKEY_LOCAL_MACHINE,
           kHKEY_USERS, kHKEY_CURRENT_CONFIG, kHKEY_DYN_DATA);
  THomepage   = string;
  TMyRegistry = class(TComponent)
  private
    { Private-Deklarationen }
    INI             : TINIFile;
    REG             : TRegistry;
    bCreateReg,                 // Create registry-string if not exist??
    ToINI,                      // (intern version)
    bSaveNew,                   // writes key if not exists
    bSaveToINI      : boolean;  // INI or Registry ?
    sAbout,
    sVersion        : string;
    sHomepage       : THomepage;
    sINIFile        : string;   // INI-Filename
    sRegString      : string;   // Registry path
    hRootKey        : HKeys;
    FOnError        : TMyRegistryError;

    procedure SetError(IsBox:Boolean;EMessage:String);
    function GetRootKey:HKey;
  protected
    { Protected-Deklarationen }
    procedure StrDummy(str:string);
  public
    { Public-Deklarationen }
    // Read-Functions
    constructor Create(AOwner:TComponent); override;
    function ReadString  (Section:string;Ident:string;Default:string):string;
    function ReadInteger (Section:string;Ident:string;Default:Integer):Integer;
    function ReadBool    (Section:string;Ident:string;Default:Boolean):Boolean;
    procedure ReadSection (const Section: string; Strings: TStrings);
    procedure ReadSectionValues(const Section: string; Strings: TStrings);

    // Write-Methods
    procedure WriteString  (Group:string;Ident:string;Value:string);
    procedure WriteInteger (Group:string;Ident:string;Value:integer);
    procedure WriteBool    (Group:string;Ident:string;Value:boolean);
    procedure DeleteKey    (Section:string;Ident:string);
    procedure Open;
    procedure Close;
    //
    function ValueExists(Section:string;Ident:string): Boolean;
  published
    { Published-Deklarationen }
    property SaveToINI : boolean read bSaveToINI write bSaveToINI default False;
    property SaveNewKey: boolean read bSaveNew   write bSaveNew;
    property INIFile   : string  read sINIFile   write sINIFile;
    property RegPath   : string  read sRegString write sRegString;
    property RootKey   : HKeys   read hRootKey   write hRootKey default kHKEY_LOCAL_MACHINE;
    property CanCreate : boolean read bCreateReg write bCreateReg default True;
    property Version   : string  read sVersion   write StrDummy;
    property eMail     : string  read sAbout     write StrDummy;
    property Homepage  : string  read sHomepage  write StrDummy;
    property OnError   : TMyRegistryError read FOnError write FOnError;
  end;

  TUpdatesProperty = class(TPropertyEditor)
  public
    procedure Edit; override;
    procedure SetValue(const Value: string); override;
    function GetValue: string; override;
    function GetAttributes: TPropertyAttributes; override;
  end;

procedure Register;

implementation
uses ShellAPI;

// ---------------------------------------------------------------------- //
// TMyRegistry.SetError
// ---------------------------------------------------------------------- //
procedure TMyRegistry.SetError(IsBox:Boolean;EMessage:string);
begin
  if Assigned(FOnError) then FOnError(Self,IsBox,EMessage) else
  if IsBox then
    MessageBox(0,PChar(EMessage),'Error',mb_Ok or mb_IconExclamation or mb_DefButton1);
end;

// ---------------------------------------------------------------------- //
// TMyRegistry.ValueExists
// ---------------------------------------------------------------------- //
function TMyRegistry.ValueExists(Section:string;Ident:string): Boolean;
begin
  if ToINI then
    result:=INI.ReadString(Section,Ident,'')<>''
  else
    with Reg do begin
      OpenKey(sRegString+Section,bCreateReg);
      try
        Result:=ValueExists(Ident);
      except
        Result:=False;
      end;
      CloseKey;
    end;
end;

procedure TMyRegistry.StrDummy(str:string);
begin
end;

// ---------------------------------------------------------------------- //
// TMyRegistry.Create
// ---------------------------------------------------------------------- //
constructor TMyRegistry.Create(AOwner:TComponent);
begin
  inherited Create(AOwner);
  sVersion  := 'v1.10';
  sAbout    := 'Martin.Zahl@Informatik.Uni-Oldenburg.de';
  sHomepage := 'http://www.Informatik.Uni-Oldenburg.de/~mzahl';
  hRootKey  := kHKEY_LOCAL_MACHINE;
  bSaveNew  := False;
  INI       := Nil;
  Reg       := Nil;
end;

// ---------------------------------------------------------------------- //
// TMyRegistry.GetRootKey
// ---------------------------------------------------------------------- //
function TMyRegistry.GetRootKey:HKey;
begin
  case hRootKey of
    kHKEY_CLASSES_ROOT    : Result:=HKEY_CLASSES_ROOT;
    kHKEY_CURRENT_USER    : Result:=HKEY_CURRENT_USER;
    kHKEY_LOCAL_MACHINE   : Result:=HKEY_LOCAL_MACHINE;
    kHKEY_USERS           : Result:=HKEY_USERS;
    kHKEY_CURRENT_CONFIG  : Result:=HKEY_CURRENT_CONFIG;
    kHKEY_DYN_DATA        : Result:=HKEY_DYN_DATA;
    else Result:=HKEY_LOCAL_MACHINE;
  end;
end;

// ---------------------------------------------------------------------- //
// TMyRegistry.Open
// ---------------------------------------------------------------------- //
procedure TMyRegistry.Open;
begin
  if bSaveToINI then try
    if Trim(sINIFile)='' then
      raise EInOutError.Create('INI file does not exist !!');
    if INI=Nil then
      INI:=TINIFile.Create(sINIFile);
  except
    on E:Exception do SetError(true,E.Message);
  end else try
    if Trim(sRegString)='' then
      raise EInOutError.Create('You must insert a registry string !!');
    if REG=Nil then
      REG:=TRegistry.Create;
    REG.RootKey:=GetRootKey;
    if sRegString[length(sRegString)]<>'\' then
      sRegString:=sRegString+'\';
  except
    on E:Exception do SetError(true,E.Message);
  end;
  ToINI:=bSaveToINI;
end;

// ---------------------------------------------------------------------- //
// TMyRegistry.Close
// ---------------------------------------------------------------------- //
procedure TMyRegistry.Close;
begin
  try
    if ToINI then begin
      if INI<>Nil then
        INI.Free;
      INI:=Nil;
    end else begin
      if REG<>Nil then
        REG.Free;
      REG:=Nil;
    end;
  except
  end;
end;

// ---------------------------------------------------------------------- //
// TMyRegistry.WriteString
// ---------------------------------------------------------------------- //
procedure TMyRegistry.WriteString(Group:string;Ident:string;Value:string);
begin
  if ToINI then
    INI.WriteString(Group,Ident,Value)
  else
    with Reg do begin
      OpenKey(sRegString+Group,bCreateReg);
      try
        WriteString(Ident,Value);
      except
        on E:Exception do SetError(true,E.Message);
      end;
      CloseKey;
    end;
end;

// ---------------------------------------------------------------------- //
// TMyRegistry.WriteInteger
// ---------------------------------------------------------------------- //
procedure TMyRegistry.WriteInteger(Group:string;Ident:string;Value:integer);
begin
  if ToINI then
    INI.WriteInteger(Group,Ident,Value)
  else
    with Reg do begin
      OpenKey(sRegString+Group,bCreateReg);
      try
        WriteInteger(Ident,Value);
      except
        on E:Exception do SetError(true,E.Message);
      end;
      CloseKey;
    end;
end;

// ---------------------------------------------------------------------- //
// TMyRegistry.WriteBool
// ---------------------------------------------------------------------- //
procedure TMyRegistry.WriteBool(Group:string;Ident:string;Value:boolean);
begin
  if ToINI then
    INI.WriteBool(Group,Ident,Value)
  else
    with Reg do begin
      OpenKey(sRegString+Group,bCreateReg);
      try
        WriteBool(Ident,Value);
      except
        on E:Exception do SetError(true,E.Message);
      end;
      CloseKey;
    end;
end;

// ---------------------------------------------------------------------- //
// TMyRegistry.ReadString
// ---------------------------------------------------------------------- //
function TMyRegistry.ReadString(Section:string;Ident:string;Default:string):string;
begin
  if ToINI then
    Result:=INI.ReadString(Section,Ident,Default)
  else
    with Reg do begin
      OpenKey(sRegString+Section,bCreateReg);
      try
        Result:=ReadString(Ident);
        if (Result='') then
          raise EInOutError.Create('Value not found !!');
      except
        on E:Exception do begin
          SetError(false,E.Message);
          Result:=Default;
        end;
      end;
      CloseKey;
    end;
  // write default-value if key not exists
  if bSaveNew and (Result=Default) then
    WriteString(Section,Ident,Default);
end;

// ---------------------------------------------------------------------- //
// TMyRegistry.ReadInteger
// ---------------------------------------------------------------------- //
function TMyRegistry.ReadInteger(Section:string;Ident:string;Default:Integer):Integer;
begin
  if ToINI then
    Result:=INI.ReadInteger(Section,Ident,Default)
  else
    with Reg do begin
      OpenKey(sRegString+Section,bCreateReg);
      try
        Result:=ReadInteger(Ident);
      except
        on E:Exception do begin
          SetError(false,E.Message);
          Result:=Default;
        end;
      end;
      CloseKey;
    end;
  // write default-value if key not exists
  if bSaveNew and (Result=Default) then
    WriteInteger(Section,Ident,Default);
end;

// ---------------------------------------------------------------------- //
// TMyRegistry.ReadBool
// ---------------------------------------------------------------------- //
function TMyRegistry.ReadBool(Section:string;Ident:string;Default:Boolean):Boolean;
begin
  if ToINI then
    Result:=INI.ReadBool(Section,Ident,Default)
  else
    with Reg do begin
      OpenKey(sRegString+Section,bCreateReg);
      try
        Result:=ReadBool(Ident);
      except
        on E:Exception do begin
          SetError(false,E.Message);
          Result:=Default;
        end;
      end;
      CloseKey;
    end;
  // write default-value if key not exists
  if bSaveNew and (Result=Default) then
    WriteBool(Section,Ident,Default);
end;

// ---------------------------------------------------------------------- //
// TMyRegistry.ReadSection
// ---------------------------------------------------------------------- //
procedure TMyRegistry.ReadSection(const Section: string; Strings: TStrings);
begin
  if ToINI then begin
    INI.ReadSection(Section,Strings);
  end else
    raise EInOutError.Create('Cannot read sections from the registry !!');
end;

// ---------------------------------------------------------------------- //
// TMyRegistry.ReadSectionValues
// ---------------------------------------------------------------------- //
procedure TMyRegistry.ReadSectionValues(const Section: string; Strings: TStrings);
begin
  if ToINI then begin
    INI.ReadSectionValues(Section,Strings);
  end else
    raise EInOutError.Create('Cannot read sections values from the registry !!');
end;

// ---------------------------------------------------------------------- //
// TMyRegistry.DeleteKey
// ---------------------------------------------------------------------- //
procedure TMyRegistry.DeleteKey(Section:string;Ident:string);
begin
  if ToINI then
    INI.DeleteKey(Section,Ident)
  else
    with Reg do begin
      OpenKey(sRegString+Section,bCreateReg);
      if not DeleteKey(Ident) then
        SetError(false,'The key '+Ident+' was not deleted');
      CloseKey;
    end;
end;

// ----- Property editor ----------------------------------------

procedure TUpdatesProperty.Edit;
begin
  ShellExecute(GetDesktopWindow(), 'open', PChar(MyHomepage), nil, nil, SW_SHOWNORMAL);
end;

function TUpdatesProperty.GetAttributes: TPropertyAttributes;
begin
  Result := [paDialog];
end;

procedure TUpdatesProperty.SetValue(const Value: string);
begin
  SetStrValue(Value);
end;

function TUpdatesProperty.GetValue: string;
begin
  Result := MyHomepage;
end;

// ---------------------------------------------------------------------- //

procedure Register;
begin
  RegisterComponents('Samples', [TMyRegistry]);
  RegisterPropertyEditor(TypeInfo(THomepage), TObject, 'Homepage', TUpdatesProperty);
end;

end.
